@@ -1,10 +1,10 @@
 ;; -*- no-byte-compile: t; lexical-binding: nil -*-
-(define-package "ducpel" "0.1"
+(define-package "ducpel" "20260204.1533"
   "Logic game with sokoban elements."
-  '((cl-lib "0.5"))
+  '((emacs "29.1"))
   :url "https://github.com/alezost/ducpel"
-  :commit "ece785baaa102bd2e9d54257af3a92bacc5757bc"
-  :revdesc "ece785baaa10"
+  :commit "bb0275a8aa7e8aa0895979711b037573544d50c1"
+  :revdesc "bb0275a8aa7e"
   :keywords '("games")
   :authors '(("Alex Kost" . "alezost@gmail.com"))
   :maintainers '(("Alex Kost" . "alezost@gmail.com")))
