@@ -1,12 +1,12 @@
-;;; ducpel.el --- Logic game with sokoban elements
+;;; ducpel.el --- Logic game with sokoban elements  -*- lexical-binding: t -*-
 
-;; Copyright (C) 2014 Alex Kost
+;; Copyright © 2014, 2026 Alex Kost
 
 ;; Author: Alex Kost <alezost@gmail.com>
 ;; Created: 31 Mar 2014
-;; Package-Version: 0.1
-;; Package-Revision: ece785baaa10
-;; Package-Requires: ((cl-lib "0.5"))
+;; Package-Version: 20260204.1533
+;; Package-Revision: bb0275a8aa7e
+;; Package-Requires: ((emacs "29.1"))
 ;; URL: https://github.com/alezost/ducpel
 ;; Keywords: games
 
@@ -14,11 +14,14 @@
 ;; it under the terms of the GNU General Public License as published by
 ;; the Free Software Foundation, either version 3 of the License, or
 ;; (at your option) any later version.
-
+;;
 ;; This program is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
 ;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;; GNU General Public License for more details.
+;;
+;; You should have received a copy of the GNU General Public License
+;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -37,12 +40,12 @@
 ;;
 ;;   (setq ducpel-levels-directory "/path/to/ducpel-levels-dir")
 
-;; After that you can "M-x ducpel" and enjoy.  Use:
+;; After that, you can call "M-x ducpel" and enjoy.  Use:
 ;;
 ;; - arrow keys to move your man;
 ;; - TAB to switch to another man;
 ;; - "u" to undo a move;
-;; - SPC to activate a special cell (currently only exit cell is special);
+;; - SPC to activate a special cell (exit or teleport);
 ;; - "R" to restart the level;
 ;; - "N"/"P"/"L" to go to the next/previous/particular level.
 
@@ -87,7 +90,8 @@ To replay the moves, use
 
 (defcustom ducpel-levels-directory
   (expand-file-name "levels"
-                    (file-name-directory (locate-library "ducpel")))
+                    (file-name-directory
+                     (file-truename load-file-name)))
   "Directory with level files."
   :type 'directory
   :group 'ducpel)
@@ -229,16 +233,21 @@ Each element of the list is a list of the form (X Y).")
 (defvar ducpel-active-man-index 0
   "Index of the active man in `ducpel-men'.")
 
+(defvar ducpel-teleports nil
+  "List of coordinates of the teleports on the current level.
+Each element of the list is a list of the form (X Y).")
+
 (defvar ducpel-undo-list nil
   "List of full undo information.
 
 Each element of the list has a form:
 
-  (CELLS MEN ACTIVE)
+  (CELLS MEN ACTIVE TELEPORTS)
 
 CELLS has a form of `ducpel-undo-current-cells'.
 MEN has a form of `ducpel-undo-current-men'.
-ACTIVE has a form of `ducpel-undo-current-active-index'.")
+ACTIVE has a form of `ducpel-undo-current-active-index'.
+TELEPORTS has a form of `ducpel-undo-current-teleports'.")
 
 (defvar ducpel-undo-current-cells nil
   "List of changes of the cells made after the last move.
@@ -261,6 +270,11 @@ it means the coordinates of the man were not changed.")
   "Index of the man that was active after the last move.
 If nil, it means the active man was not changed.")
 
+(defvar ducpel-undo-current-teleports nil
+  "List of coordinates of the teleports after the last move.
+Has a form of `ducpel-teleports'.
+If nil, it means teleports were not changed.")
+
 (defvar ducpel-moves 0
   "The number of moves for the current level.")
 
@@ -327,7 +341,7 @@ Properties in property lists:
   "Fill `ducpel-cell-plists'."
   (cl-flet ((pset (&rest plist)
                   (aset ducpel-cell-plists
-                        (apply 'ducpel-get-cell-char-by-plist plist)
+                        (apply #'ducpel-get-cell-char-by-plist plist)
                         plist)))
     (pset :type ducpel-empty)
     (pset :type ducpel-wall)
@@ -354,7 +368,7 @@ Properties in property lists:
   "Set cell at X, Y to the cell defined by property list PLIST.
 Return cell character of the set cell."
   (let* ((old-char (gamegrid-get-cell x y))
-         (new-char (apply 'ducpel-get-cell-char-by-plist plist)))
+         (new-char (apply #'ducpel-get-cell-char-by-plist plist)))
     (gamegrid-set-cell x y new-char)
     (push (list x y old-char) ducpel-undo-current-cells)
     new-char))
@@ -445,10 +459,15 @@ return nil; otherwise return index of the new active man."
                   ducpel-active-man-index))
         (setq ducpel-active-man-index index)))))
 
+(defun ducpel-get-active-cell-xy ()
+  "Return coordinates of the cell with the active man.
+Returning value is a list of the form (X Y)."
+  (aref ducpel-men ducpel-active-man-index))
+
 (defun ducpel-get-active-cell-plist ()
   "Return cell plist of the cell with the active man."
-  (apply 'ducpel-get-cell-plist-by-xy
-         (aref ducpel-men ducpel-active-man-index)))
+  (apply #'ducpel-get-cell-plist-by-xy
+         (ducpel-get-active-cell-xy)))
 
 (defun ducpel-next-man ()
   "Select next man."
@@ -493,8 +512,12 @@ Return non-nil if the action was successful."
       (ducpel-done-p t)
       (setq success t))
      ((eql floor ducpel-teleport)
-      ;; TODO implement teleporting
-      (message "This strange thing won't work."))
+      (if (null (cdr ducpel-teleports))
+          ;; If a single teleport on the map.
+          (message "This strange thing looks broken.")
+        (if (ducpel-teleport-active-man)
+            (setq success t)
+          (message "Hm, perhaps the teleport is blocked."))))
      (t (message "Nothing interesting here.")))
     success))
 
@@ -506,14 +529,53 @@ Return non-nil if the move was successful."
       (ducpel-get-man-xy)
     (ducpel-move x y direction)))
 
+(defun ducpel-teleport-active-man ()
+  "Try to teleport active man to a free teleport cell.
+If the next teleport after the current one is blocked, try the
+next after it and so on.
+Return non-nil, if teleportation was successful."
+  (let* ((active-xy (ducpel-get-active-cell-xy))
+         (next-teleports (member active-xy ducpel-teleports)))
+    (or next-teleports
+        (error "Active man is not on the teleport cell"))
+    ;; Getting next free teleport: if the rest teleports are blocked,
+    ;; continue searching from the beginning of `ducpel-teleports'.
+    (when-let* ((xy (or (ducpel-teleport-get-free-cell (cdr next-teleports))
+                        (ducpel-teleport-get-free-cell
+                         (cl-loop for teleport in ducpel-teleports
+                                  until (equal teleport active-xy)
+                                  collect teleport)))))
+      (let ((from-x (car active-xy))
+            (from-y (cadr active-xy))
+            (to-x (car xy))
+            (to-y (cadr xy)))
+        (ducpel-set-cell
+         to-x to-y
+         :type ducpel-active-man :floor ducpel-teleport)
+        (ducpel-set-cell
+         from-x from-y
+         :type ducpel-floor :floor ducpel-teleport)
+        (ducpel-set-man-xy from-x from-y to-x to-y)
+        t))))
+
+(defun ducpel-teleport-get-free-cell (cells)
+  "Return first free cell from a list of coordinates CELLS.
+Cell is free if it is a floor with no object (man or box) on it.
+Return nil if none of the cells is free."
+  (cl-loop for cell in cells
+           if (eql (plist-get
+                    (apply #'ducpel-get-cell-plist-by-xy cell)
+                    :type)
+                   ducpel-floor)
+           return cell))
+
 (defun ducpel-cell-can-move-p (floor-type direction)
   "Return non-nil, if a cell with FLOOR-TYPE can move in the DIRECTION.
 Direction should have a value of one of the following constants:
 `ducpel-left-move', `ducpel-right-move',
 `ducpel-up-move', `ducpel-down-move'."
-  (let ((moves (cdr (assoc floor-type ducpel-move-type-alist))))
-    (and moves
-         (/= 0 (logand moves direction)))))
+  (when-let* ((moves (cdr (assoc floor-type ducpel-move-type-alist))))
+    (/= 0 (logand moves direction))))
 
 (defun ducpel-get-xy (from-x from-y direction &optional val)
   "Return coordinates by shifting FROM-X, FROM-Y to the DIRECTION by VAL.
@@ -556,11 +618,13 @@ Returning value is a list of coordinates of the last empty cell."
 (defun ducpel-done-p (&optional show-message)
   "Return non-nil if current level is passed.
 If SHOW-MESSAGE is non-nil, also show a message in minibuffer."
-  (let ((done (= ducpel-done (length ducpel-men))))
-    (and done
-         show-message
-         ;; FIXME Do not hardcode the bindings
-         (message "DONE! Press 'r c' to replay, 'r s' to save, 'R' to restart, 'N' for the next level."))
+  (when-let* ((done (= ducpel-done (length ducpel-men))))
+    (when show-message
+      (message (substitute-command-keys "DONE! Press:
+\\[ducpel-replay-solution] to replay,
+\\[ducpel-save-replay] to save,
+\\[ducpel-restart-level] to restart,
+\\[ducpel-next-level] to start the next level.")))
     done))
 
 (defun ducpel-add-move ()
@@ -627,7 +691,7 @@ Return non-nil if the shift was successful, nil otherwise."
                              ducpel-from-char))
          (ducpel-from-type (plist-get ducpel-from-plist :type))
          success)
-    ;; Most cell types can't be moved
+    ;; Most cell types can't be moved.
     (unless (memql ducpel-from-type
                    (list ducpel-empty ducpel-wall
                          ducpel-impassable ducpel-floor))
@@ -648,7 +712,7 @@ Return non-nil if the shift was successful, nil otherwise."
            ((eql ducpel-from-type ducpel-man)
             (cl-incf ducpel-power)
             (when (or (ducpel-move-object-to-floor)
-                      (ducpel-move-man-to-wall))
+                      (ducpel-move-object-to-wall))
               (ducpel-set-man-xy ducpel-from-x ducpel-from-y
                                  ducpel-to-x   ducpel-to-y)
               (setq success t)))
@@ -659,7 +723,7 @@ Return non-nil if the shift was successful, nil otherwise."
             (let ((new-from-plist
                    (cond
                     ((or (ducpel-move-object-to-floor)
-                         (ducpel-move-man-to-wall))
+                         (ducpel-move-object-to-wall))
                      (list :type ducpel-floor
                            :floor (plist-get ducpel-from-plist :floor)))
                     ((ducpel-move-man-to-emty)
@@ -667,7 +731,7 @@ Return non-nil if the shift was successful, nil otherwise."
               (when new-from-plist
                 (ducpel-set-man-xy ducpel-from-x ducpel-from-y
                                    ducpel-to-x ducpel-to-y)
-                (apply 'ducpel-set-cell
+                (apply #'ducpel-set-cell
                        ducpel-from-x ducpel-from-y new-from-plist)
                 (setq success t))))
 
@@ -676,6 +740,7 @@ Return non-nil if the shift was successful, nil otherwise."
                  (> ducpel-power 0))
             (cl-decf ducpel-power)
             (when (or (ducpel-move-object-to-floor)
+                      (ducpel-move-object-to-wall)
                       (ducpel-move-box-to-empty))
               (setq success t)))))))
     success))
@@ -695,8 +760,8 @@ return non-nil."
                      :floor (plist-get ducpel-to-plist :floor)
                      :box (plist-get ducpel-from-plist :box))))
 
-(defun ducpel-move-man-to-wall ()
-  "Try to move a man to a wall.
+(defun ducpel-move-object-to-wall ()
+  "Try to move an object (man or box) to a wall.
 If the move is possible, redraw the destination cell and
 return non-nil."
   (when (and (eql ducpel-to-type ducpel-wall)
@@ -723,6 +788,11 @@ return non-nil."
 If the move is possible, redraw the destination cell and
 return non-nil."
   (when (eql ducpel-to-type ducpel-empty)
+    (when (eql (plist-get ducpel-from-plist :box)
+               ducpel-teleport)
+      (setq ducpel-undo-current-teleports ducpel-teleports)
+      (push (list ducpel-to-x ducpel-to-y)
+            ducpel-teleports))
     (ducpel-set-cell ducpel-to-x ducpel-to-y
                      :type ducpel-floor
                      :floor (plist-get ducpel-from-plist :box))))
@@ -736,6 +806,7 @@ return non-nil."
 (defun ducpel-undo-reset-current ()
   "Reset current undo data to the default values."
   (setq ducpel-undo-current-cells nil
+        ducpel-undo-current-teleports nil
         ducpel-undo-current-men (make-vector (length ducpel-men) nil)
         ducpel-undo-current-active-index nil))
 
@@ -748,39 +819,41 @@ return non-nil."
   "Add undo info about the current move to `ducpel-undo-list'."
   (push (list ducpel-undo-current-cells
               ducpel-undo-current-men
-              ducpel-undo-current-active-index)
+              ducpel-undo-current-active-index
+              ducpel-undo-current-teleports)
         ducpel-undo-list)
   (ducpel-undo-reset-current))
 
-(defun ducpel-undo-changes (cells men active)
-  "Undo changes from CELLS, MEN and ACTIVE.
+(defun ducpel-undo-changes (cells men active teleports)
+  "Undo changes from CELLS, MEN, ACTIVE and TELEPORTS.
 For the meaning of arguments, see `ducpel-undo-list'."
   (mapc (lambda (change)
-          (apply 'gamegrid-set-cell change))
+          (apply #'gamegrid-set-cell change))
         cells)
   (dotimes (i (length men))
-    (let ((man (aref men i)))
-      (and man
-           (aset ducpel-men i man))))
+    (when-let* ((man (aref men i)))
+      (aset ducpel-men i man)))
   (and active
-       (setq ducpel-active-man-index active)))
+       (setq ducpel-active-man-index active))
+  (and teleports
+       (setq ducpel-teleports teleports)))
 
 (defun ducpel-undo ()
   "Undo previous move or action."
   (interactive)
-  ;; Undo possible switching of the men made since the last move
+  ;; Undo possible switching of the men made since the last move.
   (ducpel-undo-changes ducpel-undo-current-cells
                        ducpel-undo-current-men
-                       ducpel-undo-current-active-index)
+                       ducpel-undo-current-active-index
+                       ducpel-undo-current-teleports)
   (ducpel-undo-reset-current)
-  ;; Undo the last move
-  (let ((move-changes (pop ducpel-undo-list)))
-    (when move-changes
-      (apply 'ducpel-undo-changes move-changes)
-      (ducpel-remove-move)
-      (pop ducpel-moves-history)
-      (ducpel-check-done)
-      (ducpel-print-done))))
+  ;; Undo the last move.
+  (when-let* ((move-changes (pop ducpel-undo-list)))
+    (apply #'ducpel-undo-changes move-changes)
+    (ducpel-remove-move)
+    (pop ducpel-moves-history)
+    (ducpel-check-done)
+    (ducpel-print-done)))
 
 
 ;;; Replaying
@@ -830,7 +903,7 @@ Interactively, prompt for FILE."
             "(setq ducpel-moves-history '")
     (princ ducpel-moves-history (current-buffer))
     (insert ")\n")
-    (set (make-local-variable 'version-control) 'never)
+    (setq-local version-control 'never)
     (write-file file t)))
 
 
@@ -852,7 +925,7 @@ Gamegrid specifications are lists of the form:
 They are used for `gamegrid-display-options' (see
 `gamegrid-initialize-display' for details).")
 
-;; Avoid compilation warning about `ducpel-glyphs-default'
+;; Avoid compilation warning about `ducpel-glyphs-default'.
 (declare-function ducpel-glyphs-default "ducpel-glyphs" nil)
 
 (defun ducpel-get-glyphs ()
@@ -868,7 +941,7 @@ They are used for `gamegrid-display-options' (see
         (glyph-alist (ducpel-get-glyphs)))
     (dolist (assoc glyph-alist)
       (aset options
-            (apply 'ducpel-get-cell-char-by-plist (car assoc))
+            (apply #'ducpel-get-cell-char-by-plist (car assoc))
             (cdr assoc)))
     options))
 
@@ -885,7 +958,7 @@ DY is a number of line after `ducpel-height'."
   (goto-char (point-min))
   (let ((lines (forward-line (+ ducpel-height dy)))
         (inhibit-read-only t))
-    ;; Go to the line even if it does not exist
+    ;; Go to the line even if it does not exist.
     (insert (make-string lines ?\n))
     (delete-region (point) (line-end-position))
     (insert string)
@@ -973,7 +1046,7 @@ DY is a number of line after `ducpel-height'."
 
 (defun ducpel-get-cell-char-by-map-chars (map-char obj-char)
   "Return cell type character by MAP-CHAR and OBJ-CHAR characters."
-  (apply 'ducpel-get-cell-char-by-plist
+  (apply #'ducpel-get-cell-char-by-plist
          (ducpel-get-cell-plist-by-map-chars map-char obj-char)))
 
 (defun ducpel-parse-solution ()
@@ -1013,7 +1086,7 @@ Set the following variables: `ducpel-level-data',
           (objects (ducpel-parse-map ducpel-objects-re))
           (height 0)
           (width 0))
-      ;; Define height and width of the data array
+      ;; Define height and width of the data array.
       (dolist (line map)
         (cl-incf height)
         (let ((w (length line)))
@@ -1022,7 +1095,7 @@ Set the following variables: `ducpel-level-data',
       (setq ducpel-level-data (make-vector height nil)
             ducpel-width width
             ducpel-height height)
-      ;; Fill the data array
+      ;; Fill the data array.
       (cl-loop for map-line in map
                for objects-line in objects
                for y from 0
@@ -1036,25 +1109,29 @@ Set the following variables: `ducpel-level-data',
                     (aset ducpel-level-data y line))))))
 
 (defun ducpel-init-buffer ()
-  "Fill current buffer with the level map."
+  "Fill current buffer with the level map.
+Set `ducpel-men', `ducpel-active-man-index' and
+`ducpel-teleports' variables."
   (gamegrid-init-buffer ducpel-width ducpel-height ?\s)
+  (setq ducpel-teleports nil)
   (let (men)
     (dotimes (y ducpel-height)
       (dotimes (x ducpel-width)
-        (let ((char (aref (aref ducpel-level-data y) x)))
-          (when char
-            (let* ((plist (ducpel-get-cell-plist-by-cell-char char))
-                   (type (plist-get plist :type)))
-              (cond
-               ((eql type ducpel-man)
-                (push (list x y) men))
-               ((eql type ducpel-active-man)
-                (push (list x y) men)
-                (setq ducpel-active-man-index
-                      (- (length men) 1)))))
-            (gamegrid-set-cell x y char)))))
+        (when-let* ((char (aref (aref ducpel-level-data y) x)))
+          (let* ((plist (ducpel-get-cell-plist-by-cell-char char))
+                 (type (plist-get plist :type)))
+            (cond
+             ((eql type ducpel-man)
+              (push (list x y) men))
+             ((eql type ducpel-active-man)
+              (push (list x y) men)
+              (setq ducpel-active-man-index
+                    (- (length men) 1)))
+             ((eql (plist-get plist :floor) ducpel-teleport)
+              (push (list x y) ducpel-teleports))))
+          (gamegrid-set-cell x y char))))
     (setq ducpel-men
-          (apply 'vector (nreverse men)))))
+          (apply #'vector (nreverse men)))))
 
 
 ;;; UI for levels
@@ -1144,42 +1221,43 @@ signal an error."
 
 ;;; Major mode
 
-(defvar ducpel-mode-map
-  (let ((map (make-sparse-keymap)))
-    (define-key map "R"            'ducpel-restart-level)
-    (define-key map "F"            'ducpel-load-level-from-file)
-    (define-key map "L"            'ducpel-goto-level)
-    (define-key map "N"            'ducpel-next-level)
-    (define-key map "P"            'ducpel-previous-level)
-    (define-key map "u"            'ducpel-undo)
-    (define-key map "\C-_"         'ducpel-undo)
-    (define-key map [(control ?/)] 'ducpel-undo)
-    (define-key map "\t"           'ducpel-next-man)
-    (define-key map "\e\t"         'ducpel-previous-man)
-    (define-key map [backtab]      'ducpel-previous-man)
-    (define-key map " "            'ducpel-action)
-    (define-key map "b"            'ducpel-move-left)
-    (define-key map "f"            'ducpel-move-right)
-    (define-key map "p"            'ducpel-move-up)
-    (define-key map "n"            'ducpel-move-down)
-    (define-key map [left]         'ducpel-move-left)
-    (define-key map [right]        'ducpel-move-right)
-    (define-key map [up]           'ducpel-move-up)
-    (define-key map [down]         'ducpel-move-down)
-    (define-key map "rc"           'ducpel-replay-current)
-    (define-key map "rf"           'ducpel-replay-from-file)
-    (define-key map "rS"           'ducpel-replay-solution)
-    (define-key map "rs"           'ducpel-save-replay)
-    map)
-  "Keymap for `ducpel-mode'.")
+(defvar-keymap ducpel-mode-map
+  :doc "Keymap for `ducpel-mode'."
+  "SPC" #'ducpel-action
+  "TAB" #'ducpel-next-man
+  "M-<tab>" #'ducpel-previous-man
+  "<backtab>" #'ducpel-previous-man
+  "R"   #'ducpel-restart-level
+  "F"   #'ducpel-load-level-from-file
+  "L"   #'ducpel-goto-level
+  "N"   #'ducpel-next-level
+  "P"   #'ducpel-previous-level
+  "r c" #'ducpel-replay-current
+  "r f" #'ducpel-replay-from-file
+  "r S" #'ducpel-replay-solution
+  "r s" #'ducpel-save-replay
+  "b"   #'ducpel-move-left
+  "f"   #'ducpel-move-right
+  "p"   #'ducpel-move-up
+  "n"   #'ducpel-move-down
+  "u"   #'ducpel-undo
+  "<remap> <backward-char>" #'ducpel-move-left
+  "<remap> <left-char>"     #'ducpel-move-left
+  "<remap> <forward-char>"  #'ducpel-move-right
+  "<remap> <right-char>"    #'ducpel-move-right
+  "<remap> <previous-line>" #'ducpel-move-up
+  "<remap> <next-line>"     #'ducpel-move-down
+  "<remap> <undo>"          #'ducpel-undo
+  "<remap> <save-buffer>"   #'ducpel-save-replay)
 
 (define-derived-mode ducpel-mode special-mode "Ducpel"
   "Major mode for playing ducpel.
 
 \\{ducpel-mode-map}"
-  (set (make-local-variable 'gamegrid-use-glyphs) ducpel-use-glyphs)
-  ;; hl-line disturbs if `ducpel-use-glyphs' is nil
-  (set (make-local-variable 'global-hl-line-mode) nil)
+  (setq gamegrid-use-glyphs ducpel-use-glyphs
+        show-trailing-whitespace nil)
+  ;; hl-line disturbs if `ducpel-use-glyphs' is nil.
+  (setq-local global-hl-line-mode nil)
   (gamegrid-init (ducpel-get-display-options)))
 
 ;;;###autoload
